#! /bin/bash

SCRIPT_NAME=[[`basename "$0"`]]

# This script prints out the -m*** flags used for compiling GMP
# (or exits with code 1 if an error occurred).
# Assumes that env variables CXX and COCOA_EXTLIB_DIR are set.

# taken from StackExchange 256434
is_absolute()
{
    case "$1" in
	///* | //) true;;
	//*) false;;
	/*) true;;
	*) false;;
    esac
}


if [ $# != 0 ]
then
  echo "ERROR: expected no args   $SCRIPT_NAME"
  exit 1
fi

# Check environment variables CXX and COCOA_EXTLIB_DIR
if [ -z "$CXX" ]
then
  echo "ERROR: environment variable CXX not set.   $SCRIPT_NAME"
  exit 1
fi

if [ -z "$COCOA_EXTLIB_DIR" ]
then
    echo "ERROR: environment variable COCOA_EXTLIB_DIR not set.   $SCRIPT_NAME"
    exit 1
fi

# The following is a cryptic if...then block
is_absolute "$COCOA_EXTLIB_DIR" ||
(
  echo "ERROR: environment variable COCOA_EXTLIB_DIR is not absolute: \"$COCOA_EXTLIB_DIR\"   $SCRIPT_NAME"
  exit 1
)

if [ \! -d "$COCOA_EXTLIB_DIR" -o \! -d "$COCOA_EXTLIB_DIR/include" -o \! -d "$COCOA_EXTLIB_DIR/lib" ]
then
  echo "ERROR: environment variable COCOA_EXTLIB_DIR is implausible: \"$COCOA_EXTLIB_DIR\"   $SCRIPT_NAME"
  exit 1
fi


# Below we create a small C++ program for printing out the GMP compilation flags.
TODAY=`date "+%Y-%m-%d"`
TIME=`date "+%H:%M:%S"`
TMP_DIR=/tmp/CoCoALib-config-$USER-$TODAY/gmp-cxx-flags-$TIME-$$
/bin/rm -rf $TMP_DIR  &&  /bin/mkdir -p $TMP_DIR
if [ $? -ne 0 ]
then
  echo "ERROR: failed to create temporary directory \"$TMP_DIR\"   $SCRIPT_NAME"
  exit 1
fi

cd $TMP_DIR
/bin/cat > prog.C <<EOF
#include "gmp.h"
#include <iostream>

int main()
{
  std::cout << __GMP_CFLAGS << std::endl;
}
EOF

$CXX -I"$COCOA_EXTLIB_DIR/include" prog.C -o prog  > LogFile 2>&1
GMP_CXXFLAGS=`./prog 2>> LogFile`
if [ $? -ne 0 ]
then
  # Deliberately leave $TMP_DIR to assist debugging.
  echo "ERROR: test program crashed!   $SCRIPT_NAME"
  exit 1
fi

# GMP_CXXFLAGS contains all the compilation flags used when building GMP.
# We pick out just the compilation options which begin with -m
COCOALIB_CXXFLAGS=
for opt in $GMP_CXXFLAGS
do
  case $opt in
  ( -m* )
    COCOALIB_CXXFLAGS="$COCOALIB_CXXFLAGS $opt";;
  esac
done

#######################################################
# Check that these GMP CXXFLAGS actually work (they may not if the gmp.h and libgmp.a
# have been copied from another machine, e.g. installing via "brew" on MacOSX)
# See redmine 975.

$CXX $COCOALIB_CXXFLAGS -I"$COCOA_EXTLIB_DIR/include" prog.C -o prog  >> LogFile 2>&1
if [ $? -ne 0 ]
then
  COCOALIB_CXXFLAGS=""
fi

# Clean up TMP_DIR
cd # Leave TMP_DIR
/bin/rm -rf $TMP_DIR

echo $COCOALIB_CXXFLAGS
