//   Copyright (c)  2016 John Abbott, Anna M. Bigatti, Elisa Palezzato

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/MatrixOperations.H"
#include "CoCoA/CanonicalHom.H"
#include "CoCoA/DenseMatrix.H"
#include "CoCoA/QuotientRing.H" // for CanonicalRepr / DefiningIdeal
#include "CoCoA/IntOperations.H"
#include "CoCoA/MatrixView.H"
#include "CoCoA/NumTheory.H" // for NextPrime
#include "CoCoA/PPMonoidHom.H"  // for PPMonoidHom, GeneralHom
#include "CoCoA/PolyRing.H"
#include "CoCoA/RingFp.H"  // for NewRingFp
#include "CoCoA/RingHom.H"
#include "CoCoA/RingQQ.H"  // for IsQQ
#include "CoCoA/RingZZ.H"
#include "CoCoA/SparsePolyRing.H" // for HasGBasis
#include "CoCoA/VectorOperations.H" // for HasUniqueOwner
#include "CoCoA/apply.H"
#include "CoCoA/assert.H"
#include "CoCoA/bool3.H"
#include "CoCoA/convert.H"
#include "CoCoA/error.H"
#include "CoCoA/geobucket.H"
#include "CoCoA/interrupt.H" // for CheckForInterrupt
#include "CoCoA/time.H"
#include "CoCoA/utils.H" // for len
#include "CoCoA/verbose.H"  // for VerboseLog


#include <algorithm>
using std::min;
#include <iostream>
using std::ostream;
#include <limits>
using std::numeric_limits;
//#include <vector>
using std::vector;

namespace CoCoA
{

  namespace // anonymous
  {

  class LinDepMill
  {
  public:
    LinDepMill(const ring& k, long dim);
    void myAppendVec(std::vector<RingElem> v);
    const std::vector<RingElem>& myLinReln() {return myLinRelnValue;}
  
  private:
    ring myRing;
    int myVecLen;
    std::vector<std::vector<RingElem> > myM;
    std::vector<std::vector<RingElem> > myRowRepr;
    std::vector<int> myColIndices;
    std::vector<RingElem> myLinRelnValue;
  };

    LinDepMill::LinDepMill(const ring& K, long dim):
      myRing(K),
      myVecLen(dim)
    {
      if (!IsField(K)) CoCoA_ERROR(ERR::NotField, "LinDepMill ctor");
      if (dim <= 0) CoCoA_ERROR(ERR::NotPositive, "LinDepMill ctor");
      myM.reserve(dim);
      myRowRepr.reserve(dim);
      myColIndices.reserve(dim);
      myLinRelnValue.reserve(dim);
    }

  
  //  const std::vector<RingElem>& LinDepMill::myAppendVec(std::vector<RingElem> v)
  void LinDepMill::myAppendVec(std::vector<RingElem> v)
    {
      const long n = len(v);
      if (n != myVecLen) CoCoA_ERROR(ERR::IncompatDims, "LinDepMill::myAppendVec");
      if (owner(v[0]) != myRing || !HasUniqueOwner(v))
        CoCoA_ERROR(ERR::MixedRings, "LinDepMill::myAppendVec");
      const long ncols = len(myM);
      myLinRelnValue.resize(ncols, zero(myRing));
      myLinRelnValue.push_back(one(myRing));
      for (long i=0; i < ncols; ++i)
      {
        const long col = myColIndices[i];
        if (IsZero(v[col])) continue;
        const RingElem c = v[col]/myM[i][col];
        for (long j=0; j < n; ++j)
          v[j] -= c*myM[i][j];
        for (long j=0; j < len(myRowRepr[i]); ++j)
          myLinRelnValue[j] -= c*myRowRepr[i][j];
      }
      for (long i=0; i < n; ++i)
      {
        if (IsZero(v[i])) continue;
        // if v!=0 add it to the mill:
        myM.push_back(v);
        myColIndices.push_back(i);
        myRowRepr.push_back(myLinRelnValue);
        myLinRelnValue.clear();
        break;
      }
      // v is reduced: if 0 -> myLinRelnValue; else -> mill
    }


    RingElem CoeffOfTerm(ConstRefRingElem f, ConstRefPPMonoidElem pp)
    {
      for (SparsePolyIter itf=BeginIter(f); !IsEnded(itf); ++itf)
        if (PP(itf) == pp) return coeff(itf);
      return zero(CoeffRing(owner(f)));
    }


    void coefficients(vector<RingElem>& coeffs, ConstRefRingElem p, const vector<PPMonoidElem>& QB)
    {
      if (len(coeffs)!=len(QB))
        CoCoA_ERROR("len(coeffs)!=len(QB)","coefficients");
      for (long i=0; i<len(QB); ++i)  coeffs[i] = CoeffOfTerm(p, QB[i]);
    }

    void CoefficientsInMatRow(MatrixView& M, long j, 
                           ConstRefRingElem p, const vector<PPMonoidElem>& QB)
    {
      if (NumCols(M)!=len(QB))
        CoCoA_ERROR("NumCols(M)!=len(QB)","CoefficientsInMatRow");
      for (long i=0; i<NumRows(M); ++i) SetEntry(M,i,j, CoeffOfTerm(p ,QB[i]));
    }


    void MatrixFlatten(vector<RingElem>& v, ConstMatrixView M)
    {
      if (len(v) != NumRows(M)*NumCols(M))
        CoCoA_ERROR("len(v) != NumRows(M)*NumCols(M)","MatrixFlatten");
      long k = -1;
      for (long i=0; i<NumRows(M); ++i)
        for (long j=0; j<NumCols(M); ++j)
          v[++k] = M(i,j);
    }


    long position(ConstRefPPMonoidElem elem, vector<PPMonoidElem> QB)
    {
      //for (long i=1; i<len(QB); ++i)  if (elem==QB[i]) return i;
      // CoCoA_ERROR(ERR::SERIOUS, "position"); // only called with elem in QB
      for (long i=len(QB)-1; i>=0; --i)  if (elem==QB[i]) return i;
      return 0;  // just to keep compiler quiet
    }


    matrix MultiplicationMat(ConstRefRingElem f_orig, const ideal& I)
    {
      ring P = RingOf(I);
      if (P != owner(f_orig))
        CoCoA_ERROR("arguments must be in the same ring", "MultiplicationMat");
      if (!IsZeroDim(I))
        CoCoA_ERROR("ideal must be 0-dimensional", "MultiplicationMat");
      const vector<PPMonoidElem>& x_PPM = indets(PPM(P));
      const vector<RingElem>& x_P = indets(P);
      vector<PPMonoidElem> QB = QuotientBasis(I);
      RingElem f = NF(f_orig, I);
      RingElem p;
      matrix M = NewDenseMat(CoeffRing(P), len(QB), len(QB));
      for (long j=0; j<len(QB); ++j)
      {
        if (deg(QB[j])<12)
          p = NF(f*monomial(P, QB[j]), I);
        else
        {
          //std::cout << "*";
          long k = NumIndets(P);
          for (--k; k>=0; --k)  if (IsDivisible(QB[j], x_PPM[k])) break;
          long quot_pos = position(QB[j]/x_PPM[k], QB);
          RingElem NFquot = zero(P); // scalar product:
          for (long d=0; d<NumRows(M); ++d)
            NFquot += monomial(P, M(d,quot_pos), QB[d]);
          p = NF(NFquot*x_P[k], I);
        }
        for (long i=0; i<len(QB); ++i)  SetEntry(M,i,j, CoeffOfTerm(p,QB[i]));
      }
      return M;
    }


    RingElem NFPower(ConstRefRingElem t, long exp, const ideal& I) 
    {
      //// ring compatible?
      if (exp<0) CoCoA_ERROR(ERR::NotNonNegative, "NFPower");
      if (exp==0) return one(owner(t));
      if (exp==1) return t;
      if (IsEven(exp))
          return NFPower(NF(power(t,2), I), exp/2, I);
      return NF(NFPower(NF(power(t,2), I), exp/2, I)*t, I);
    }
    

    namespace // anonymous
    {
      
      matrix ChangeBasis(const ideal& I, const std::vector<PPMonoidElem>& QB2)
      {
        const std::vector<PPMonoidElem> QB = QuotientBasis(I);
        const ring P = RingOf(I);
        matrix M = NewDenseMat(CoeffRing(P), len(QB), len(QB));
        for (long j=0; j<len(QB2); ++j)
        {
          RingElem p = NF(monomial(P, QB2[j]), I);
          for (long i=0; i<len(QB); ++i)  SetEntry(M,i,j, CoeffOfTerm(p,QB[i]));
        }
        if (IsZero(det(M))) CoCoA_ERROR("QB2 is not a basis", "ChangeBasis");
        return M;
      }

    }



  } // anonymous namespace


  matrix FrobeniusMat(const ideal& I)
  {
    VerboseLog VERBOSE("FrobeniusMat");
    ring P = RingOf(I);
    long p = ConvertTo<long>(characteristic(P));
    if (p==0) CoCoA_ERROR("Characteristic must be finite", "FrobeniusMat");
    long e = LogCardinality(CoeffRing(P));
    if (p==0) CoCoA_ERROR("Field must be finite", "FrobeniusMat");

    vector<PPMonoidElem> QB = QuotientBasis(I);
    matrix M = NewDenseMat(ZeroMat(CoeffRing(P), len(QB), len(QB)));
    SetEntry(M, 0,0, one(CoeffRing(P)));

    RingElem NFj(P);
    RingElem NFk(P);
    RingElem NFquot(P);
    double U = CpuTime();
    for (long j=1; j<len(QB); ++j)
    {
      long k = j-1;
      for (; k>0; --k)  if (IsDivisible(QB[j], QB[k])) break;
      if (k==0)
        NFj = NFPower(monomial(P, QB[j]), p, I);
      else
      {
        long quot_pos = position(QB[j]/QB[k], QB);
        NFk = zero(P);
        NFquot = zero(P);
        for (long d=0; d<NumRows(M); ++d)
        { // scalar products:
          NFk    += monomial(P, M(d,k),        QB[d]);
          NFquot += monomial(P, M(d,quot_pos), QB[d]);
        }
        NFj = NF(NFk*NFquot, I);
      }
      // coefficients
      //      for (long i=0; i<NumRows(M); ++i) SetEntry(M,i,j, CoeffOfTerm(NFj,QB[i]));
      CoefficientsInMatRow(M, j, NFj, QB);
    }
    VERBOSE(10) << "FrobeniusMat: cycle --> " << CpuTime() - U << std::endl;
    return power(M, e);
  }


  matrix FrobeniusMat(const ideal& I, const std::vector<PPMonoidElem>& QB2)
  {
    matrix FM = FrobeniusMat(I);
    matrix BC = ChangeBasis(I,QB2);
    return inverse(BC)*FM*BC;
  }
  

  RingElem MinPoly(ConstMatrixView M, ConstRefRingElem x)
  {
    if (NumRows(M)!=NumCols(M))
      CoCoA_ERROR(ERR::NotSquareMatrix, "MinPoly");
    if (!IsIndet(x))
      CoCoA_ERROR("2nd arg must be an indeterminate","MinPoly");
  
    ring RM = RingOf(M);
    ring Rx = owner(x);

    if (RM==Rx)
      // TODO: convert M into matrix of coefficients (if possible)
      CoCoA_ERROR(ERR::NYI, "MinPoly");
    ring K = CoeffRing(Rx);
    //   If (IsZZ(R) or IsQQ(R)) Then
    //     M := apply(CanonicalHom(R, K), M);
    //     R := K;
    //   EndIf;
    if (K!=RM)
      CoCoA_ERROR(ERR::MixedRings, "MinPoly");
    long NRows = NumRows(M);
 
    LinDepMill ILD(K, NRows*NRows);    
    vector<RingElem> flattenv(NRows*NRows, zero(K));

    matrix MD = NewDenseMat(IdentityMat(K, NRows));
    MatrixFlatten(flattenv, MD);
    ILD.myAppendVec(flattenv);

    double S1=0, S2=0, S5=0, V;
    while ( (ILD.myLinReln()).empty() )
    {
      V = CpuTime(); MD = M*MD;                   S1 += (CpuTime()-V);
      V = CpuTime(); MatrixFlatten(flattenv, MD); S2 += (CpuTime()-V);
      V = CpuTime(); ILD.myAppendVec(flattenv);   S5 += (CpuTime()-V);
    }

    double Y = CpuTime();
    RingHom phi = CoeffEmbeddingHom(Rx);
    RingElem mi = zero(owner(x));
    const vector<RingElem>& linreln = ILD.myLinReln();
    for (long d=0; d<len(linreln); ++d)  mi += phi(linreln[d])*power(x,d);
    Y = CpuTime() - Y;

    return mi;
  }


//   RingElem MinPolyQuotMat2(ConstRefRingElem L, const ideal& I, ConstRefRingElem x)
//   {
//     return MinPoly(MultiplicationMat(L,I), x);
//   }


  RingElem MinPolyQuotMat(ConstRefRingElem f_orig, const ideal& I, ConstRefRingElem x)
  {
    VerboseLog VERBOSE("MinPolyQuotMat");
    VERBOSE(3) << f_orig << std::endl;
    double U;
    RingElem f = NF(f_orig, I);
    U = CpuTime();
    matrix M = MultiplicationMat(f,I);
    VERBOSE(10) << "Tempo MultiplicationMat: " << (CpuTime() - U) << std::endl;

    U = CpuTime();
    
    //  If not(IsIndet(X)) Then error("MinPoly: 2nd arg must be an indeterminate"); EndIf;
  
    ring R = RingOf(M);
  
    //if R = RingOf(X) then return MinPolyInKx(M, X); endif;
  
    ring K = CoeffRing(owner(x));
    vector<PPMonoidElem> QB = QuotientBasis(I);
    LinDepMill ILD(K, len(QB));
    //  If R <> K Then error("MinPoly(M,x): rings of M and x are not compatible"); EndIf;

    vector<RingElem> FlatVec(len(QB), zero(K));
    matrix fPow = NewDenseMat(ZeroMat(K, len(QB), 1));
    SetEntry(fPow, 0,0, one(K));
    
    MatrixFlatten(FlatVec, fPow);
    ILD.myAppendVec(FlatVec);

    U = CpuTime();
    double S1 = 0;
    double S2 = 0;
    double S5 = 0;  
    double V;
    while ( (ILD.myLinReln()).empty() )
    {
      V = CpuTime(); fPow = M*fPow;                S1 += (CpuTime()-V);
      V = CpuTime(); MatrixFlatten(FlatVec, fPow); S2 += (CpuTime()-V);
      V = CpuTime(); ILD.myAppendVec(FlatVec);     S5 += (CpuTime()-V);
    }
    VERBOSE(10) << "tempo del ciclo: --> " << (CpuTime() - U) << std::endl;

    double Y = CpuTime();
    RingHom phi = CoeffEmbeddingHom(owner(x));
    //ScalarProduct(col, [x^d | d in 0..i-1]);
    RingElem mp = zero(owner(x));
    const vector<RingElem>& linreln = ILD.myLinReln();
    for (long d=0; d<len(linreln); ++d)  mp += phi(linreln[d])*power(x,d);
    Y = CpuTime() - Y;

    VERBOSE(10) << "  tempo di: M*fPow --> " << S1 << std::endl
                << "  tempo di: flatten --> " << S2 << std::endl
                << "  tempo di: LinDepMill --> " << S5 << std::endl
                << "Post-ciclo --> " << Y << std::endl;
    return mp;
  }


RingElem MinPolyQuotElim(ConstRefRingElem L, const ideal& I, ConstRefRingElem x)
{
  VerboseLog VERBOSE("MinPolyQuotElim");
  VERBOSE(3) << "  --MinPolyElim: " << L << std::endl;
  // controllare anelli
  ring P = RingOf(I);
  long n = NumIndets(P);
  //  myIndets = concat(IndetSymbols(P), [Record [head ="aux", indices =[]]]);
  ring S = NewPolyRing(CoeffRing(P), n+1);
  vector<RingElem> indetS_1;
  for (long i=0; i<n; ++i) indetS_1.push_back(indet(S,i));
  RingHom phi = PolyAlgebraHom(P, S, indetS_1);
  vector<RingElem> zero_x;
  for (long i=0; i<n; ++i) zero_x.push_back(zero(P));
  zero_x.push_back(x);
  RingHom psi = PolyAlgebraHom(S, P, zero_x);
  RingElem s = indet(S, n); // auxiliar variable
  ideal J = ideal(apply(phi, gens(I))) + ideal(s-phi(L));
  MakeUnique(J)->myElim(indetS_1);
  return psi(monic(gens(J)[0]));
}


RingElem MinPolyQuotDef(ConstRefRingElem f_orig, const ideal& I, ConstRefRingElem x)
{
  VerboseLog VERBOSE("MinPolyQuotDef");
  //  const double T = CpuTime();
  ring P = owner(f_orig);
  ring P_x = owner(x);
  ring K = CoeffRing(P);
  // fare tutti i controlli sugli anelli
  if (K != CoeffRing(P_x))
    CoCoA_ERROR("incompatible coeff rings", "MinPolyDefQuot");
  if (IsOne(I))
    CoCoA_ERROR("Ideal is (1)", "MinPolyDefQuot");
  RingElem f = NF(f_orig, I);
  vector<PPMonoidElem> QB = QuotientBasis(I);
  LinDepMill ILD(K, len(QB));
  VERBOSE(3) << f_orig << std::endl;
  vector<RingElem> coeffs(len(QB), zero(K));
  
  RingElem p = one(P);
  coeffs[0] = one(K);
  ILD.myAppendVec(coeffs);

  double W = CpuTime();
  double S1 = 0;
  double S2 = 0;
  double S3 = 0;
  double S4 = 0;
  double U;
  while ( (ILD.myLinReln()).empty() )
  {
    U = CpuTime();  p *= f;  S1 = S1+(CpuTime()-U);
    U = CpuTime();  p = NF(p, I);  S2 = S2+(CpuTime()-U);
    U = CpuTime();  coefficients(coeffs, p, QB);  S3 = S3+(CpuTime()-U);
    U = CpuTime();  ILD.myAppendVec(coeffs);  S4 = S4+(CpuTime()-U);
  }
  W = CpuTime() - W;
  VERBOSE(10) << "  ... ciclo " << W << std::endl;

  double Y = CpuTime();
  RingHom phi = CoeffEmbeddingHom(P_x);
  //ScalarProduct(col, [x^d | d in 0..i-1]);
  RingElem mp = zero(P_x);
  const vector<RingElem>& linreln = ILD.myLinReln();
  for (long d=0; d<len(linreln); ++d)  mp += phi(linreln[d])*power(x,d);  
  Y = CpuTime() - Y;

  VERBOSE(10)  << "Tempo interno al ciclo --> " << S1+S2+S3+S4 << std::endl
               << "  tempo di: p*f --> " << S1 << std::endl
               << "  tempo di: NF --> " << S2 << std::endl
               << "  tempo di: coefficients --> " << S3 << std::endl
               << "  tempo di: LinDepMill --> " << S4 << std::endl
               << "Post-ciclo --> " << Y << std::endl;
  return monic(mp);
}


RingElem MinPolyQuotDefLin(ConstRefRingElem f_orig, const ideal& I, ConstRefRingElem x)
{
  VerboseLog VERBOSE("MinPolyQuotDefLin");
  //  const double T = CpuTime();
  ring P = owner(f_orig);
  ring P_x = owner(x);
  ring K = CoeffRing(P);
  RingHom psi = CoeffEmbeddingHom(P);
  // fare tutti i controlli sugli anelli
  if (K != CoeffRing(P_x))
    CoCoA_ERROR("incompatible coeff rings", "MinPolyDefQuot");
  if (IsOne(I))
    CoCoA_ERROR("Ideal is (1)", "MinPolyDefQuot");
  RingElem f = NF(f_orig, I);
  vector<PPMonoidElem> QB = QuotientBasis(I);
  LinDepMill ILD(K, len(QB));

  VERBOSE(3) << "  --MinPolyDefQuot: " << f_orig << std::flush;
  //---- "MultiplicationMat"  fxQB
  double U = CpuTime();
  const vector<RingElem>& x_P = indets(P);
  const vector<PPMonoidElem>& x_PPM = indets(PPM(P));
  vector<RingElem> fxQB(len(QB));
  for (long j=0; j<len(QB); ++j)
  {
    if (deg(QB[j])<12)
      fxQB[j] = NF(f*monomial(P, QB[j]), I);
    else
    {
      VERBOSE(3) << "*";
      long k = NumIndets(P);
      for (--k; k>=0; --k)  if (IsDivisible(QB[j], x_PPM[k])) break;
      fxQB[j] = NF(fxQB[position(QB[j]/x_PPM[k], QB)]*x_P[k], I);
    }
  }
  VERBOSE(10) << "Tempo MultiplicationMat: " << (CpuTime() - U) << std::endl;
  //---- MultiplicationMat - end

  vector<RingElem> coeffs(len(QB), zero(K));
  
  RingElem p = one(P);
  coeffs[0] = one(K);
  ILD.myAppendVec(coeffs);
  // std::cout << "Pre-ciclo: " << (CpuTime() - T) << std::endl;
  double W = CpuTime();
  double S1 = 0;
  double S2 = 0;
  double S3 = 0;
  double S4 = 0;
  while ( (ILD.myLinReln()).empty() )
  {
    U = CpuTime();  //p = p*f;
    RingElem partial = zero(P);
    for (SparsePolyIter itp=BeginIter(p); !IsEnded(itp); ++itp)
      partial += fxQB[position(PP(itp), QB)]*psi(coeff(itp));
    p = partial;
    S1 = S1+(CpuTime()-U);
    //    U = CpuTime();  p = NF(p, I);  S2 = S2+(CpuTime()-U);
    U = CpuTime();  coefficients(coeffs, p, QB);  S3 = S3+(CpuTime()-U);
    U = CpuTime();  ILD.myAppendVec(coeffs);  S4 = S4+(CpuTime()-U);
  }
  W = CpuTime() - W;
  VERBOSE(10) << "  ... MinPolyDefQuot: ciclo " << W << std::endl;

  double Y = CpuTime();
  RingHom phi = CoeffEmbeddingHom(P_x);
  //ScalarProduct(col, [x^d | d in 0..i-1]);
  RingElem mp = zero(P_x);
  const vector<RingElem>& linreln = ILD.myLinReln();
  for (long d=0; d<len(linreln); ++d)  mp += phi(linreln[d])*power(x,d);  
  Y = CpuTime() - Y;

  VERBOSE(10)  << "Tempo interno al ciclo --> " << S1+S2+S3+S4 << std::endl
               << "  tempo di: p*f --> " << S1 << std::endl
               << "  tempo di: NF --> " << S2 << std::endl
               << "  tempo di: coefficients --> " << S3 << std::endl
               << "  tempo di: LinDepMill --> " << S4 << std::endl
               << "Post-ciclo --> " << Y << std::endl;
  return monic(mp);
}

RingElem MinPolyModular(ConstRefRingElem f, const ideal& I, ConstRefRingElem x);
  

RingElem MinPolyQuot(ConstRefRingElem f, const ideal& I, ConstRefRingElem x)
{
  if (IsQQ(CoeffRing(owner(f))))
    return MinPolyModular(f, I, x);
  return MinPolyQuotDef(f, I, x);
}


  //--------- MinPoly in QuotientRing -----------------------

RingElem MinPolyMat(ConstRefRingElem f, ConstRefRingElem x)
{
  if (!IsQuotientRing(owner(f)))
    CoCoA_ERROR(ERR::NotElemQuotientRing, "MinPoly for RingElem");
  return MinPolyQuotMat(CanonicalRepr(f), DefiningIdeal(owner(f)), x);
}

RingElem MinPolyDef(ConstRefRingElem f, ConstRefRingElem x)
{
  if (!IsQuotientRing(owner(f)))
    CoCoA_ERROR(ERR::NotElemQuotientRing, "MinPoly for RingElem");
  return MinPolyQuotDef(CanonicalRepr(f), DefiningIdeal(owner(f)), x);
}

RingElem MinPolyElim(ConstRefRingElem f, ConstRefRingElem x)
{
  if (!IsQuotientRing(owner(f)))
    CoCoA_ERROR(ERR::NotElemQuotientRing, "MinPoly for RingElem");
  return MinPolyQuotElim(CanonicalRepr(f), DefiningIdeal(owner(f)), x);
}


RingElem MinPoly(ConstRefRingElem f, ConstRefRingElem x)
{ return MinPolyDef(f, x); }


//---------- MinPoly modular -----------------------------------------

  namespace //  anonymous
  {
    RingElem UniPolyToQQ(ConstRefRingElem f, ConstRefRingElem x)
    {
      const SparsePolyRing QQx = owner(x);
      const SparsePolyRing Fpx(owner(f));
      const PPMonoidHom psi(GeneralHom(PPM(Fpx), std::vector<PPMonoidElem>(NumIndets(Fpx),LPP(x))));
      RingElem ans(QQx);
      for (SparsePolyIter it=BeginIter(f) ; !IsEnded(it) ; ++it )
        ans += monomial(QQx, ConvertTo<BigInt>(coeff(it)), psi(PP(it)));
      return ans;
    }
  }


//define MinPolyModular(f, I, x, opt vrb, opt primes)
RingElem MinPolyModular(ConstRefRingElem f, const ideal& I, ConstRefRingElem x)
{
  VerboseLog VERBOSE("MinPolyModular");
  const SparsePolyRing QQx = RingOf(I);
  if (!IsQQ(CoeffRing(QQx)))
    CoCoA_ERROR("CoeffRing for I must be QQ", "MinPolyModular");
  if (!IsQQ(CoeffRing(owner(x))))
    CoCoA_ERROR("CoeffRing for indet must be QQ", "MinPolyModular");
  long PrimeCounter = 1;
  std::vector<RingElem> G;
  if (HasGBasis(I))
  {
    if (!IsZeroDim(I)) CoCoA_ERROR("non zero-dim", "MinPolyModular");
    //    G = ReducedGBasis(I);
    G = GBasis(I);
  }
  else
  {
    VERBOSE(1) << "WARNING: no precomputed GBasis, using gens!" << std::endl;
    G = gens(I);
  }
  long p = NextPrime(32009);
  VERBOSE(1) << PrimeCounter << ": prime is " << p << std::endl;
  SparsePolyRing Fpx = NewPolyRing(NewRingFp(p), NewSymbols(NumIndets(QQx)));
  RingHom pi = PolyRingHom(QQx, Fpx, CanonicalHom(RingQQ(), Fpx), indets(Fpx));
  ideal Ip = ideal(apply(pi, G));
  if ((!HasGBasis(I)) && (!IsZeroDim(Ip)))
    CoCoA_ERROR("non zero-dim", "MinPolyModular");
  RingElem xFp = indet(Fpx,0);
  RingElem mpCRT = UniPolyToQQ(MinPolyQuot(pi(f), Ip, xFp), x);
  VERBOSE(100) << "mpCRT = " << mpCRT << std::endl;
  RingElem mp;
  BigInt mpModulus(p);
  //  println "residue poly is ", LM(mpCRT), "+ ...";
  while (true)
  {
    CheckForInterrupt("MinPolyModular");
    ++PrimeCounter;
    p = NextPrime(p);
    VERBOSE(1) << PrimeCounter << ": prime is " << p << std::endl;
    Fpx = NewPolyRing(NewRingFp(p), NewSymbols(NumIndets(QQx)));
    VERBOSE(100) << "owner(xFp) = " << owner(xFp) << std::endl;
    pi = PolyRingHom(QQx, Fpx, CanonicalHom(RingQQ(), Fpx), indets(Fpx));
    VERBOSE(100) << "prima di apply " << std::endl;
    VERBOSE(100) << "apply " << apply(pi,G) << std::endl;
    ideal Jp = ideal(apply(pi, G));
    VERBOSE(100) << "RingOf(Jp) = " << RingOf(Jp) << std::endl;
    if ((!HasGBasis(I)) && (!IsZeroDim(Jp)))
      CoCoA_ERROR("non zero-dim", "MinPolyModular");
    VERBOSE(100) << "owner(pi(f)) = " << owner(pi(f)) << std::endl;
    mp = MinPolyQuot(pi(f), Jp, indet(Fpx,0));
    VERBOSE(100) << "mp = " << mp << std::endl;
    if (!HasGBasis(I))
      CRTPoly(mpCRT,mpModulus,  mpCRT,mpModulus,  UniPolyToQQ(mp,x),BigInt(p));
    else
    {
      if (deg(mp) == deg(mpCRT))
        CRTPoly(mpCRT,mpModulus, mpCRT,mpModulus, UniPolyToQQ(mp,x),BigInt(p));
      else
      {
        if (deg(mp) < deg(mpCRT))
          VERBOSE(2) << "DISCARDED: deg(mp) < deg(mpCRT)" << std::endl;
        else
        {
          VERBOSE(2) << "SWAPPED: deg(mp) > deg(mpCRT)" << std::endl;
          mpModulus = p;
          mpCRT = UniPolyToQQ(mp,x);
        }
      }
    }
    VERBOSE(100) << "mpCRT = " << mpCRT << std::endl;
    try
    {
      const RingElem Rat_mp = RatReconstructPoly(mpCRT, mpModulus);
      return Rat_mp;
      }
      catch (const CoCoA::ErrorInfo& e)
      {
//        if (message(e) != "cannot reconstruct rational")
//          std::cout << "---- "<< message(e) << std::endl;
      }
  } // while true
}



  //--------- ShapeLemma -----------------------

/// seems to work
std::vector<RingElem> ShapeLemma(const ideal& I)
{
  ring P = RingOf(I);
  ring K = CoeffRing(P);
  RingHom phi = CoeffEmbeddingHom(P);
  // fare tutti i controlli sugli anelli
  if (IsOne(I))
    CoCoA_ERROR("Ideal is (1)", "ShapeLemma");
  if (!IsZeroDim(I))
    CoCoA_ERROR("ideal must be 0-dimensional", "ShapeLemma");
  vector<PPMonoidElem> QB = QuotientBasis(I);
  vector<RingElem> coeffs(len(QB), zero(K));
  LinDepMill ILD(K, len(QB));
  
  RingElem z = indet(P, NumIndets(P)-1);
  RingElem p = one(P);
  coeffs[0] = one(K);
  ILD.myAppendVec(coeffs);
  while ( (ILD.myLinReln()).empty() )
  {
    p = NF(p*z, I);
    coefficients(coeffs, p, QB);
    ILD.myAppendVec(coeffs);
  }
  if (len(ILD.myLinReln())!=len(QB)+1)
    CoCoA_ERROR("last indet not good: len(linreln)!=len(QB)", "ShapeLemma");
  RingElem MinPoly_z(P);
  const vector<RingElem>& linreln = ILD.myLinReln();
  for (long d=0; d<len(linreln); ++d)  MinPoly_z += phi(linreln[d])*power(z,d);
  std::vector<RingElem> FirstIndetsExpr;
  for (long i=0; i<NumIndets(P)-1; ++i)
  {
    p = NF(indet(P, i), I);
    coefficients(coeffs, p, QB);
    ILD.myAppendVec(coeffs);
    RingElem f(P);
    const vector<RingElem>& linreln = ILD.myLinReln();
    for (long d=0; d<len(linreln)-1; ++d)  f += phi(linreln[d])*power(z,d);
    f += phi(linreln[len(linreln)-1])*indet(P,i);
    FirstIndetsExpr.push_back(f);
  }
  FirstIndetsExpr.push_back(MinPoly_z);
  return FirstIndetsExpr;
}


} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/MinPoly.C,v 1.19 2017/02/16 12:41:40 bigatti Exp $
// $Log: MinPoly.C,v $
// Revision 1.19  2017/02/16 12:41:40  bigatti
// -- highlighted code for CoefficientsInMatRow
//
// Revision 1.18  2017/02/06 16:08:34  bigatti
// -- just some verbosity
//
// Revision 1.17  2017/02/01 09:48:04  bigatti
// -- added verbosity printing
//
// Revision 1.16  2016/12/06 16:08:23  bigatti
// -- improved MinPolyModular
//
// Revision 1.15  2016/11/25 17:06:53  bigatti
// -- increased verbosity for some debugging entires
//
// Revision 1.14  2016/11/23 13:33:11  bigatti
// -- MinPolyModular called automatically by MinPolyQuot
// -- MinPolyModular with verbose and interrupt
//
// Revision 1.13  2016/10/27 13:07:24  bigatti
// -- added MinPolyQuotDefLin
// -- cleaned up code, unified notation
//
// Revision 1.12  2016/10/24 12:10:11  bigatti
// -- added ShapeLemma (first draft)
//
// Revision 1.11  2016/10/10 16:22:15  bigatti
// -- renamed myColTbl --> myColIndices
// -- aligned code for timings in MinPoly functions
// -- added check for ideal(1)
//
// Revision 1.10  2016/09/22 14:13:46  bigatti
// -- added safety checks in FrobeniusMat
//
// Revision 1.9  2016/09/08 14:13:42  bigatti
// -- using now HasUniqueOwner
//
// Revision 1.8  2016/06/27 14:50:28  bigatti
// -- now FrobeniusMat may take two args
//
// Revision 1.7  2016/06/20 15:24:42  bigatti
// -- renamed MinPolyXX --> MinPolyQuotXX
// -- added MinPolyDef(f, x)
// -- added default to MinPolyDef in both cases
//
// Revision 1.6  2016/06/14 08:26:01  bigatti
// -- added MINPOLY_DEBUG flag for enabling debugging info
//
// Revision 1.5  2016/06/10 15:50:42  bigatti
// -- new version of FrobeniusMat
//
// Revision 1.4  2016/06/06 11:45:42  bigatti
// -- added initialization for timer
//
// Revision 1.3  2016/04/14 11:35:32  bigatti
// -- added FrobeniusMat (by Elisa Palezzato)
//
// Revision 1.2  2016/03/18 15:13:47  abbott
// Summary: Commented out two unused variables (for timing)
//
// Revision 1.1  2016/03/18 11:54:00  bigatti
// -- first import
//
